==> webform_handler_defer-1.1.0.tar.gz: webform_handler_defer/src/Plugin/WebformHandler/WebformHandlerDefer.php <==
<?php

namespace Drupal\webform_handler_defer\Plugin\WebformHandler;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\webform\Plugin\WebformHandlerBase;
use Drupal\webform\WebformSubmissionInterface;
use Drupal\webform_handler_defer\WebformDeferredProcessor;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Handler which defers processing of other handlers.
 *
 * @WebformHandler(
 *   id = "webform_handler_defer",
 *   label = @Translation("Webform Handler Defer"),
 *   category = @Translation("Custom"),
 *   description = @Translation("Defers specified handlers until cron execution."),
 *   cardinality = \Drupal\webform\Plugin\WebformHandlerInterface::CARDINALITY_SINGLE,
 *   results = \Drupal\webform\Plugin\WebformHandlerInterface::RESULTS_PROCESSED,
 * )
 */
class WebformHandlerDefer extends WebformHandlerBase {

  /**
   * Logger service.
   */
  protected LoggerChannelInterface $logger;

  /**
   * Webform Handler Defer processor.
   */
  protected WebformDeferredProcessor $processor;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->logger = $container->get('logger.factory')->get('webform_handler_defer');
    $instance->processor = $container->get('webform_handler_defer.processor');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'defer_handlers' => [],
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form['defer_handlers'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Handlers to defer'),
      '#description' => $this->t(
        'Selected handlers are processed later, rather than on form submission. Note that some handlers may not be compatible.'
      ),
      '#options' => $this->processor->setWebform($this->getWebform())->getAvailableHandlers(),
      '#default_value' => $this->configuration['defer_handlers'],
    ];
    // Remove our own handler from the options list.
    unset($form['defer_handlers']['#options'][$this->getHandlerId()]);

    return parent::buildConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->configuration['defer_handlers'] = array_filter($form_state->getValue('defer_handlers'));
    parent::submitConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function getSummary() {
    return [
      '#markup' => $this->t('Deferred: @handlers', [
        '@handlers' => implode(', ', $this->configuration['defer_handlers']),
      ]),
    ];
  }

  /**
   * {@inheritdoc}
   *
   * Trigger our handler on submission method 'postSave'.
   *
   * @see webform/src/PluginWebformHandlerBase.php 'Submission methods'
   *
   * @todo Consider possible need to intercept events other than 'postSave'.
   */
  public function postSave(WebformSubmissionInterface $submission, $update = TRUE) {
    // Only process new submissions, not updates.
    if ($update) {
      return;
    }

    // Log submission and defer handlers, check and report on failure.
    $defer_handlers = array_keys($this->configuration['defer_handlers']);
    if (!$this->processor->setup($submission, $defer_handlers)->defer()) {
      // Failed: log a warning and continue - handlers will run normally.
      $this->logger->warning("Submission {$submission->id()} handlers were not deferred.");
    }
  }

}

==> webform_handler_defer-1.1.1.tar.gz: webform_handler_defer/src/Plugin/WebformHandler/WebformHandlerDefer.php <==
<?php

namespace Drupal\webform_handler_defer\Plugin\WebformHandler;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\webform\Plugin\WebformHandlerBase;
use Drupal\webform\WebformSubmissionInterface;
use Drupal\webform_handler_defer\WebformDeferredProcessor;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Handler which defers processing of other handlers.
 *
 * @WebformHandler(
 *   id = "webform_handler_defer",
 *   label = @Translation("Webform Handler Defer"),
 *   category = @Translation("Custom"),
 *   description = @Translation("Defers the specified handlers until cron execution."),
 *   cardinality = \Drupal\webform\Plugin\WebformHandlerInterface::CARDINALITY_SINGLE,
 *   results = \Drupal\webform\Plugin\WebformHandlerInterface::RESULTS_PROCESSED,
 * )
 */
class WebformHandlerDefer extends WebformHandlerBase {

  /**
   * Logger service.
   */
  protected LoggerChannelInterface $logger;

  /**
   * Webform Handler Defer processor.
   */
  protected WebformDeferredProcessor $processor;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->logger = $container->get('logger.factory')->get('webform_handler_defer');
    $instance->processor = $container->get('webform_handler_defer.processor');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'defer_handlers' => [],
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    // Get list of submission handlers, remove our own handler from the list.
    $handlers = [];
    foreach ($this->getWebform()->getHandlers() as $id => $handler) {
      $handlers[$id] = $handler->label();
    }
    unset($handlers[$this->getHandlerId()]);

    // Define our form element.
    $form['defer_handlers'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Handlers to defer'),
      '#description' => $this->t(
        'Selected handlers are processed later, rather than on form submission. Note that some handlers may not be compatible.'
      ),
      '#options' => $handlers,
      '#default_value' => $this->configuration['defer_handlers'],
    ];
    // Ensure that our handler runs first.
    $this->setWeight(-PHP_INT_MAX);

    return parent::buildConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->configuration['defer_handlers'] = array_filter($form_state->getValue('defer_handlers'));
    parent::submitConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function getSummary() {
    return [
      '#markup' => $this->t('Deferred: @handlers', [
        '@handlers' => implode(', ', $this->configuration['defer_handlers']),
      ]),
    ];
  }

  /**
   * {@inheritdoc}
   *
   * Trigger our handler on submission method 'postSave'.
   *
   * @see webform/src/PluginWebformHandlerBase.php
   *
   * @todo Consider possible need to intercept events other than 'postSave'.
   */
  public function postSave(WebformSubmissionInterface $submission, $update = TRUE) {
    // Only process new submissions, not updates.
    if ($update) {
      return;
    }

    // Log submission and defer handlers, check and report on failure.
    $defer_handlers = array_keys($this->configuration['defer_handlers']);
    if (!$this->processor->setup($submission, $defer_handlers)->defer()) {
      // Failed: log a warning and continue - handlers will run normally.
      $this->logger->warning("Submission {$submission->id()} handlers were not deferred.");
    }
  }

}
